@@ -5,5 +5,5 @@ if {![info exists ::env(TREECTRL_LIBRARY)]
     append script "[list set ::treectrl_library $dir]
 "
 }
-append script [list load [file join $dir libtreectrl23.so.0.0] treectrl]
-package ifneeded treectrl 2.3 $script
+append script [list load [file join $dir libtreectrl24.so.0.0] treectrl]
+package ifneeded treectrl 2.4.1 $script
