@@ -1,10 +1,10 @@
 #! /bin/sh
 # the next line restarts with tclsh \
-exec tclsh8.5 "$0" ${1+"$@"}
+exec /usr/local/bin/tclsh8.5 "$0" ${1+"$@"}
 
-# RCS: @(#) $Id: demo.tcl,v 1.93 2010/11/29 04:31:37 treectrl Exp $
+# Copyright (c) 2002-2011 Tim Baker
 
-set VERSION 2.3
+set VERSION 2.4.1
 
 package require Tk 8.4
 
@@ -183,12 +183,14 @@ if {$tile} {
     set checkbuttonCmd ::ttk::checkbutton
     set radiobuttonCmd ttk::radiobutton
     set scrollbarCmd ::ttk::scrollbar
+    set scaleCmd ::ttk::scale
 } else {
     set entryCmd ::entry
     set buttonCmd ::button
     set checkbuttonCmd ::checkbutton
     set radiobuttonCmd ::radiobutton
     set scrollbarCmd ::scrollbar
+    set scaleCmd ::scale
 }
 
 option add *TreeCtrl.Background white
@@ -203,7 +205,7 @@ if {$tile} {
 	    set font {Geneva 9}
 	}
 	macosx {
-	    set font {{Lucida Grande} 11}
+	    set font {{Lucida Grande} 13}
 	}
 	unix {
 	    set font {Helvetica -12}
@@ -264,6 +266,7 @@ foreach file {
     gradients
     gradients2
     gradients3
+    headers
     help
     imovie
     layout
@@ -273,6 +276,7 @@ foreach file {
     outlook-newgroup
     random
     span
+    table
     textvariable
     www-options
 } {
@@ -316,11 +320,11 @@ proc MakeMenuBar {} {
     } else {
 #	uplevel #0 source ~/Programming/console.tcl
     }
-    $m2 add command -label "Event Browser" -command ToggleEventsWindow
-    $m2 add command -label "Identify" -command ToggleIdentifyWindow
+    $m2 add command -label "Event Browser" -command EventsWindow::ToggleWindowVisibility
+    $m2 add command -label "Identify" -command IdentifyWindow::ToggleWindowVisibility
     $m2 add command -label "Style Editor" -command ToggleStyleEditorWindow
-    $m2 add command -label "View Source" -command ToggleSourceWindow
-    $m2 add command -label "Magnifier" -command ToggleLoupeWindow
+    $m2 add command -label "View Source" -command SourceWindow::ToggleWindowVisibility
+    $m2 add command -label "Magnifier" -command LoupeWindow::ToggleWindowVisibility
     $m2 add separator
     $m2 add checkbutton -label "Native Gradients" -command ToggleNativeGradients \
 	-variable ::NativeGradients
@@ -348,13 +352,15 @@ proc MakeMenuBar {} {
 		-variable ::DemoTheme -value $theme
 	}
 	$m2 add separator
-	$m2 add command -label "Inspector" -command ToggleThemeWindow
+	$m2 add command -label "Inspector" -command ThemeWindow::ToggleWindowVisibility
     }
-    
+
     return
 }
 
-proc MakeEventsWindow {} {
+namespace eval EventsWindow {}
+
+proc EventsWindow::Init {} {
     set w [toplevel .events]
     wm withdraw $w
 #    wm transient $w .
@@ -367,7 +373,7 @@ proc MakeEventsWindow {} {
     $m1 add cascade -label "Dynamic" -menu [menu $m1.m2 -tearoff 0]
     $m1 add command -label "Clear Window" -command "$w.f.t item delete all" \
 	-accelerator Ctrl+X
-    $m1 add command -label "Rebuild Menus" -command "RebuildEventsMenus $w.f.t $m"
+    $m1 add command -label "Rebuild Menus" -command "EventsWindow::RebuildMenus $w.f.t $m"
     $m add cascade -label "Events" -menu $m1
 
     bind $w <Control-KeyPress-x> "$w.f.t item delete all"
@@ -401,9 +407,9 @@ proc MakeEventsWindow {} {
 
     $T column configure C0 -itemstyle s1
 
-    RebuildEventsMenus $T $m
+    RebuildMenus $T $m
 
-    wm protocol $w WM_DELETE_WINDOW "ToggleEventsWindow"
+    wm protocol $w WM_DELETE_WINDOW "EventsWindow::ToggleWindowVisibility"
     switch -- $::thisPlatform {
 	macintosh -
 	macosx {
@@ -416,7 +422,9 @@ proc MakeEventsWindow {} {
 
     return
 }
-proc RebuildEventsMenus {T m} {
+
+proc EventsWindow::RebuildMenus {T m} {
+    variable Priv
     foreach event [lsort -dictionary [[DemoList] notify eventnames]] {
 	set details [lsort -dictionary [[DemoList] notify detailnames $event]]
 	foreach detail $details {
@@ -438,35 +446,43 @@ proc RebuildEventsMenus {T m} {
     set menu(static) $m.m1.m1
     set menu(dynamic) $m.m1.m2
     foreach {pattern linkage} $patterns {
-	if {![info exists ::EventTrack($pattern)]} {
-	    set ::EventTrack($pattern) 1
+	if {![info exists Priv(track,$pattern)]} {
+	    set Priv(track,$pattern) 1
 	}
 	$menu($linkage) add checkbutton -label $pattern \
-	    -variable EventTrack($pattern) \
-	    -command [list ToggleEvent $T $pattern]
+	    -variable ::EventsWindow::Priv(track,$pattern) \
+	    -command [list EventsWindow::ToggleEvent $T $pattern]
     }
     foreach linkage {static dynamic} {
 	$menu($linkage) add separator
 	$menu($linkage) add command -label "Toggle All" \
-	    -command [list ToggleEvents $T $patterns2($linkage)]
+	    -command [list EventsWindow::ToggleEvents $T $patterns2($linkage)]
     }
 
-    set ::Events {}
-    set ::EventsId ""
+    set Priv(events) {}
+    set Priv(afterId) ""
     foreach {pattern linkage} $patterns {
 	[DemoList] notify bind $T $pattern {
-	    lappend Events %?
-	    if {$EventsId eq ""} {
-		set EventsId [after idle [list RecordEvents %W]]
-	    }
+	    EventsWindow::EventBinding %W %?
 	}
     }
     return
 }
-proc RecordEvents {T} {
-    set ::EventsId ""
-    set events $::Events
-    set ::Events {}
+
+proc EventsWindow::EventBinding {T charMap} {
+    variable Priv
+    lappend Priv(events) $charMap
+    if {$Priv(afterId) eq ""} {
+	set Priv(afterId) [after idle [list EventsWindow::RecordEvents $T]]
+    }
+    return
+}
+
+proc EventsWindow::RecordEvents {T} {
+    variable Priv
+    set Priv(afterId) ""
+    set events $Priv(events)
+    set Priv(events) {}
     if {![winfo ismapped .events]} return
     if {[$T item numchildren root] > 2000} {
 	set N [expr {[$T item numchildren root] - 2000}]
@@ -487,7 +503,8 @@ proc RecordEvents {T} {
     $T see "last visible"
     return
 }
-proc RecordEvent {T list open} {
+
+proc EventsWindow::RecordEvent {T list open} {
     set I [$T item create -open $open]
     array set map $list
     $T item text $I C0 $map(P)
@@ -502,10 +519,11 @@ proc RecordEvent {T list open} {
     }
     return
 }
-proc ToggleEventsWindow {} {
+
+proc EventsWindow::ToggleWindowVisibility {} {
     set w .events
     if {![winfo exists $w]} {
-	MakeEventsWindow
+	Init
     }
     if {[winfo ismapped $w]} {
 	wm withdraw $w
@@ -515,48 +533,78 @@ proc ToggleEventsWindow {} {
     }
     return
 }
-proc ToggleEvent {T pattern} {
-    [DemoList] notify configure $T $pattern -active $::EventTrack($pattern)
-    return    
-}
-proc ToggleEvents {T patterns} {
-    foreach pattern $patterns {
-	set ::EventTrack($pattern) [expr {!$::EventTrack($pattern)}]
-	ToggleEvent $T $pattern
-    }
-    return    
+
+proc EventsWindow::ToggleEvent {T pattern} {
+    variable Priv
+    [DemoList] notify configure $T $pattern -active $Priv(track,$pattern)
+    return
 }
 
-proc MakeIdentifyWindow {} {
+proc EventsWindow::ToggleEvents {T patterns} {
+    variable Priv
+    foreach pattern $patterns {
+	set Priv(track,$pattern) [expr {!$Priv(track,$pattern)}]
+	ToggleEvent $T $pattern
+    }
+    return
+}
+
+namespace eval IdentifyWindow {}
+
+proc IdentifyWindow::Init {} {
     set w .identify
     toplevel $w
     wm withdraw $w
     wm title $w "TkTreeCtrl Identify"
     set wText $w.text
-    text $wText -state disabled -width 50 -height 2 -font [[DemoList] cget -font]
+    text $wText -state disabled -width 70 -height 3 -font [[DemoList] cget -font]
     $wText tag configure tagBold -font DemoFontBold
     pack $wText -expand yes -fill both
-    wm protocol $w WM_DELETE_WINDOW "ToggleIdentifyWindow"
+    wm protocol $w WM_DELETE_WINDOW "IdentifyWindow::ToggleWindowVisibility"
     return
 }
-proc UpdateIdentifyWindow {T x y} {
+
+proc IdentifyWindow::Update {T x y} {
     set w .identify
     if {![winfo exists $w]} return
+    if {![winfo ismapped $w]} return
     set wText $w.text
     $wText configure -state normal
     $wText delete 1.0 end
     set nearest [$T item id [list nearest $x $y]]
     $wText insert end "x=" tagBold "$x  " {} "y=" tagBold "$y  " {} "nearest=" tagBold $nearest\n
+    $wText insert end "string: "
     foreach {key val} [$T identify $x $y] {
 	$wText insert end $key tagBold " $val "
     }
+    $wText insert end "\narray: "
+    $T identify -array id $x $y
+    switch -- $id(where) {
+	"header" {
+	    set keys [list where header column element side]
+	}
+	"item" {
+	    set keys [list where item column element button line]
+	}
+	default {
+	    set keys [array names id]
+	}
+    }
+    foreach key $keys {
+	set val $id($key)
+	if {$val eq ""} {
+	    set val "\"\""
+	}
+	$wText insert end $key tagBold " $val "
+    }
     $wText configure -state disabled
     return
 }
-proc ToggleIdentifyWindow {} {
+
+proc IdentifyWindow::ToggleWindowVisibility {} {
     set w .identify
     if {![winfo exists $w]} {
-	MakeIdentifyWindow
+	Init
     }
     if {[winfo ismapped $w]} {
 	wm withdraw $w
@@ -567,7 +615,9 @@ proc ToggleIdentifyWindow {} {
     return
 }
 
-proc MakeSourceWindow {} {
+namespace eval SourceWindow {}
+
+proc SourceWindow::Init {} {
     set w [toplevel .source]
     wm withdraw $w
 #    wm transient $w .
@@ -599,7 +649,7 @@ proc MakeSourceWindow {} {
     grid configure $f.sh -row 1 -column 0 -sticky we
     grid configure $f.sv -row 0 -column 1 -sticky ns
 
-    wm protocol $w WM_DELETE_WINDOW "ToggleSourceWindow"
+    wm protocol $w WM_DELETE_WINDOW "SourceWindow::ToggleWindowVisibility"
     switch -- $::thisPlatform {
 	macintosh -
 	macosx {
@@ -612,7 +662,8 @@ proc MakeSourceWindow {} {
 
     return
 }
-proc ShowSource {file} {
+
+proc SourceWindow::ShowSource {file} {
     wm title .source "TkTreeCtrl Source: $file"
     set path [Path $file]
     set t .source.f.t
@@ -623,7 +674,8 @@ proc ShowSource {file} {
     close $chan
     return
 }
-proc ToggleSourceWindow {} {
+
+proc SourceWindow::ToggleWindowVisibility {} {
     set w .source
     if {[winfo ismapped $w]} {
 	wm withdraw $w
@@ -650,7 +702,8 @@ proc ToggleStyleEditorWindow {} {
     return
 }
 
-proc MakeThemeWindow {} {
+namespace eval ThemeWindow {}
+proc ThemeWindow::Init {} {
     set w [toplevel .theme]
     wm withdraw $w
 #    wm transient $w .
@@ -659,7 +712,7 @@ proc MakeThemeWindow {} {
     set m [menu $w.menubar]
     $w configure -menu $m
     set m1 [menu $m.m1 -tearoff 0]
-    $m1 add command -label "Set List" -command SetThemeWindow
+    $m1 add command -label "Set List" -command ThemeWindow::SetList
     $m add cascade -label "Theme" -menu $m1
 
     TreePlusScrollbarsInAFrame $w.f 1 1
@@ -681,16 +734,17 @@ proc MakeThemeWindow {} {
 
     $T column configure C0 -itemstyle s1
 
-    SetThemeWindow
+    SetList
 
-    wm protocol $w WM_DELETE_WINDOW "ToggleThemeWindow"
+    wm protocol $w WM_DELETE_WINDOW "ThemeWindow::ToggleWindowVisibility"
 
     return
 }
-proc ToggleThemeWindow {} {
+
+proc ThemeWindow::ToggleWindowVisibility {} {
     set w .theme
     if {![winfo exists $w]} {
-	MakeThemeWindow
+	Init
     }
     if {[winfo ismapped $w]} {
 	wm withdraw $w
@@ -700,7 +754,8 @@ proc ToggleThemeWindow {} {
     }
     return
 }
-proc SetThemeWindow {} {
+
+proc ThemeWindow::SetList {} {
     set w .theme
     set T $w.f.t
 
@@ -759,7 +814,7 @@ proc ToggleNativeGradients {} {
     return
 }
 
-MakeSourceWindow
+SourceWindow::Init
 MakeMenuBar
 
 # http://wiki.tcl.tk/950
@@ -783,6 +838,9 @@ proc TreePlusScrollbarsInAFrame {f h v} {
 	frame $f -borderwidth 1 -relief sunken
     }
     treectrl $f.t -highlightthickness 0 -borderwidth 0
+    if {[Platform unix]} {
+	$f.t configure -headerfont [$f.t cget -font]
+    }
     $f.t configure -xscrollincrement 20 -xscrollsmoothing 1
 #    $f.t configure -itemprefix item# -columnprefix column#
     $f.t debug configure -enable no -display yes -erasecolor pink \
@@ -829,7 +887,7 @@ proc TreePlusScrollbarsInAFrame {f h v} {
 
     MakeListPopup $f.t
     MakeHeaderPopup $f.t
-    
+
     switch -- $::thisPlatform {
 	macintosh -
 	macosx {
@@ -926,7 +984,7 @@ proc MakeMainWindow {} {
 	    set x %x
 	    set y %y
 	}
-	UpdateIdentifyWindow [DemoList] $x $y
+	IdentifyWindow::Update [DemoList] $x $y
     }
     AddBindTag [DemoList] TagIdentify
 
@@ -957,9 +1015,11 @@ proc MakeMainWindow {} {
     # are generated by the library scripts.
 
     [DemoList] notify install <Header-invoke>
+    [DemoList] notify install <Header-state>
 
     [DemoList] notify install <ColumnDrag-begin>
     [DemoList] notify install <ColumnDrag-end>
+    [DemoList] notify install <ColumnDrag-indicator>
     [DemoList] notify install <ColumnDrag-receive>
 
     [DemoList] notify install <Drag-begin>
@@ -971,6 +1031,8 @@ proc MakeMainWindow {} {
     [DemoList] notify install <Edit-accept>
     ###
 
+    # This event is generated when a column's visibility is changed by
+    # the context menu.
     [DemoList] notify install <DemoColumnVisibility>
 
     return
@@ -1032,8 +1094,11 @@ proc MakeListPopup {T} {
     $m add checkbutton -label "Button Tracking" -variable Popup(buttontracking) \
 	-command {$Popup(T) configure -buttontracking $Popup(buttontracking)}
 
-    set m2 [menu $m.mVisible -tearoff no]
-    $m add cascade -label Columns -menu $m2
+    set m2 [menu $m.mColumns -tearoff no]
+    $m add cascade -label "Columns" -menu $m2
+
+    set m2 [menu $m.mHeaders -tearoff no]
+    $m add cascade -label "Headers" -menu $m2
 
     set m2 [menu $m.mColumnResizeMode -tearoff no]
     $m2 add radiobutton -label proxy -variable Popup(columnresizemode) -value proxy \
@@ -1130,70 +1195,138 @@ if 0 {
 }
 
 proc MakeHeaderPopup {T} {
-    set m [menu $T.mHeader -tearoff no]
+    set m [menu $T.mColumn -tearoff no]
 
-    set m2 [menu $m.mArrow -tearoff no]
-    $m add cascade -label Arrow -menu $m2
+    ### Header
+
+    set m1 [menu $m.mHeader -tearoff no]
+    $m add cascade -label "Header" -menu $m1
+
+    $m1 add checkbutton -label "Visible" -variable Popup(header,visible) \
+	-command [list eval $T header configure \$Popup(header) -visible \$Popup(header,visible)]
+
+    set m2 [menu $m1.mDnD -tearoff no]
+    $m1 add cascade -label "Drag and Drop" -menu $m2
+    $m2 add checkbutton -label "Draw" -variable Popup(header,drag,draw) \
+	-command [list eval $T header dragconfigure \$Popup(header) -draw \$Popup(header,drag,draw)]
+    $m2 add checkbutton -label "Enable" -variable Popup(header,drag,enable) \
+	-command [list eval $T header dragconfigure \$Popup(header) -enable \$Popup(header,drag,enable)]
+
+    ### Header column
+
+    set m1 [menu $m.mHeaderColumn -tearoff no]
+    $m add cascade -label "Header Column" -menu $m1
+
+    set m2 [menu $m1.mArrow -tearoff no]
+    $m1 add cascade -label Arrow -menu $m2
     $m2 add radiobutton -label "None" -variable Popup(arrow) -value none \
-	-command {$Popup(T) column configure $Popup(column) -arrow none}
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -arrow none}
     $m2 add radiobutton -label "Up" -variable Popup(arrow) -value up \
-	-command {$Popup(T) column configure $Popup(column) -arrow up}
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -arrow up}
     $m2 add radiobutton -label "Down" -variable Popup(arrow) -value down \
-	-command {$Popup(T) column configure $Popup(column) -arrow down}
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -arrow down}
     $m2 add separator
     $m2 add radiobutton -label "Side Left" -variable Popup(arrow,side) -value left \
-	-command {$Popup(T) column configure $Popup(column) -arrowside left}
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -arrowside left}
     $m2 add radiobutton -label "Side Right" -variable Popup(arrow,side) -value right \
-	-command {$Popup(T) column configure $Popup(column) -arrowside right}
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -arrowside right}
     $m2 add separator
     $m2 add radiobutton -label "Gravity Left" -variable Popup(arrow,gravity) -value left \
-	-command {$Popup(T) column configure $Popup(column) -arrowgravity left}
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -arrowgravity left}
     $m2 add radiobutton -label "Gravity Right" -variable Popup(arrow,gravity) -value right \
-	-command {$Popup(T) column configure $Popup(column) -arrowgravity right}
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -arrowgravity right}
 
-    $m add checkbutton -label "Button" -variable Popup(button) \
-	-command {$Popup(T) column configure $Popup(column) -button $Popup(button)}
-    $m add checkbutton -label "Expand" -variable Popup(expand) \
-	-command {$Popup(T) column configure $Popup(column) -expand $Popup(expand)}
+    $m1 add checkbutton -label "Button" -variable Popup(button) \
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -button $Popup(button)}
 
-    set m2 [menu $m.mItemJustify -tearoff no]
-    $m add cascade -label "Item Justify" -menu $m2
-    $m2 add radiobutton -label "Left" -variable Popup(itemjustify) -value left \
-	-command {$Popup(T) column configure $Popup(column) -itemjustify left}
-    $m2 add radiobutton -label "Center" -variable Popup(itemjustify) -value center \
-	-command {$Popup(T) column configure $Popup(column) -itemjustify center}
-    $m2 add radiobutton -label "Right" -variable Popup(itemjustify) -value right \
-	-command {$Popup(T) column configure $Popup(column) -itemjustify right}
-    $m2 add radiobutton -label "Unspecified" -variable Popup(itemjustify) -value none \
-	-command {$Popup(T) column configure $Popup(column) -itemjustify {}}
+    set m2 [menu $m1.mJustify -tearoff no]
+    $m1 add cascade -label "Justify" -menu $m2
+    $m2 add radiobutton -label "Left" -variable Popup(header,justify) -value left \
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -justify left}
+    $m2 add radiobutton -label "Center" -variable Popup(header,justify) -value center \
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -justify center}
+    $m2 add radiobutton -label "Right" -variable Popup(header,justify) -value right \
+	-command {$Popup(T) header configure $Popup(header) $Popup(column) -justify right}
 
-    set m2 [menu $m.mJustify -tearoff no]
-    $m add cascade -label Justify -menu $m2
-    $m2 add radiobutton -label "Left" -variable Popup(justify) -value left \
-	-command {$Popup(T) column configure $Popup(column) -justify left}
-    $m2 add radiobutton -label "Center" -variable Popup(justify) -value center \
-	-command {$Popup(T) column configure $Popup(column) -justify center}
-    $m2 add radiobutton -label "Right" -variable Popup(justify) -value right \
-	-command {$Popup(T) column configure $Popup(column) -justify right}
+    set m2 [menu $m1.mSpan -tearoff no]
+    $m1 add cascade -label Span -menu $m2
 
-    set m2 [menu $m.mLock -tearoff no]
-    $m add cascade -label Lock -menu $m2
-    $m2 add radiobutton -label "Left" -variable Popup(lock) -value left \
-	-command {$Popup(T) column configure $Popup(column) -lock left}
-    $m2 add radiobutton -label "None" -variable Popup(lock) -value none \
-	-command {$Popup(T) column configure $Popup(column) -lock none}
-    $m2 add radiobutton -label "Right" -variable Popup(lock) -value right \
-	-command {$Popup(T) column configure $Popup(column) -lock right}
+    ### Tree column
+    $m add command -label "Column"
 
-    $m add checkbutton -label "Resize" -variable Popup(resize) \
-	-command {$Popup(T) column configure $Popup(column) -resize $Popup(resize)}
-    $m add checkbutton -label "Squeeze" -variable Popup(squeeze) \
-	-command {$Popup(T) column configure $Popup(column) -squeeze $Popup(squeeze)}
-    $m add checkbutton -label "Tree Column" -variable Popup(treecolumn) \
-	-command {$Popup(T) configure -treecolumn [expr {$Popup(treecolumn) ? $Popup(column) : ""}]}
     return
 }
 
+proc MakeHeaderSubmenu {T H parentMenu} {
+
+    ### Header
+
+    set m1 [menu $parentMenu.mHeader$H -tearoff no]
+
+    $m1 add checkbutton -label "Visible" -variable Popup(header,visible,$H) \
+	-command [list eval $T header configure $H -visible \$Popup(header,visible,$H)]
+
+    return $m1
+}
+
+proc MakeColumnSubmenu {T C parentMenu {menuName ""}} {
+
+    ### Tree-column
+if 1 {
+    if {$menuName ne ""} {
+	set m1 [menu $parentMenu.mColumn$menuName -tearoff no]
+    } else {
+	set m1 [menu $parentMenu.mColumn$C -tearoff no]
+    }
+} else {
+    set m1 $parentMenu.mColumn$C
+    $m1 delete 0 end
+}
+    $m1 add checkbutton -label "Expand" -variable Popup(column,expand,$C) \
+	-command [list eval $T column configure $C -expand \$Popup(column,expand,$C)]
+
+    set m2 [menu $m1.mItemJustify -tearoff no]
+    $m1 add cascade -label "Item Justify" -menu $m2
+    $m2 add radiobutton -label "Left" -variable Popup(column,itemjustify,$C) -value left \
+	-command [list $T column configure $C -itemjustify left]
+    $m2 add radiobutton -label "Center" -variable Popup(column,itemjustify,$C) -value center \
+	-command [list $T column configure $C -itemjustify center]
+    $m2 add radiobutton -label "Right" -variable Popup(column,itemjustify,$C) -value right \
+	-command [list $T column configure $C -itemjustify right]
+    $m2 add radiobutton -label "Unspecified" -variable Popup(column,itemjustify,$C) -value none \
+	-command [list $T column configure $C -itemjustify {}]
+
+    set m2 [menu $m1.mJustify -tearoff no]
+    $m1 add cascade -label "Justify" -menu $m2
+    $m2 add radiobutton -label "Left" -variable Popup(column,justify,$C) -value left \
+	-command [list $T column configure $C -justify left]
+    $m2 add radiobutton -label "Center" -variable Popup(column,justify,$C) -value center \
+	-command [list $T column configure $C -justify center]
+    $m2 add radiobutton -label "Right" -variable Popup(column,justify,$C) -value right \
+	-command [list $T column configure $C -justify right]
+
+    set m2 [menu $m1.mLock -tearoff no]
+    $m1 add cascade -label Lock -menu $m2
+    $m2 add radiobutton -label "Left" -variable Popup(column,lock,$C) -value left \
+	-command [list $T column configure $C -lock left]
+    $m2 add radiobutton -label "None" -variable Popup(column,lock,$C) -value none \
+	-command [list $T column configure $C -lock none]
+    $m2 add radiobutton -label "Right" -variable Popup(column,lock,$C) -value right \
+	-command [list $T column configure $C -lock right]
+
+    $m1 add checkbutton -label "Resize" -variable Popup(column,resize,$C) \
+	-command [list eval $T column configure $C -resize \$Popup(column,resize,$C)]
+    $m1 add checkbutton -label "Squeeze" -variable Popup(column,squeeze,$C) \
+	-command [list eval $T column configure $C -squeeze \$Popup(column,squeeze,$C)]
+    $m1 add checkbutton -label "Tree Column" -variable Popup(column,treecolumn,$C) \
+	-command [list eval $T configure -treecolumn "\[expr {\$Popup(column,treecolumn,$C) ? $C : {}}\]"]
+    $m1 add checkbutton -label "Visible" -variable Popup(column,visible,$C) \
+	-command [list eval $T column configure $C -visible \$Popup(column,visible,$C) \; \
+	    TreeCtrl::TryEvent $T DemoColumnVisibility {} [list C $C] ]
+
+    return $m1
+}
+
 proc AddBindTag {w tag} {
 
     if {[lsearch -exact [bindtags $w] $tag] == -1} {
@@ -1212,23 +1345,53 @@ InitPics sky feather
 proc ShowPopup {T x y X Y} {
     global Popup
     set Popup(T) $T
-    set id [$T identify $x $y]
-    if {$id ne ""} {
-	if {[lindex $id 0] eq "header"} {
-	    set Popup(column) [lindex $id 1]
-	    set Popup(arrow) [$T column cget $Popup(column) -arrow]
-	    set Popup(arrow,side) [$T column cget $Popup(column) -arrowside]
-	    set Popup(arrow,gravity) [$T column cget $Popup(column) -arrowgravity]
-	    set Popup(button) [$T column cget $Popup(column) -button]
-	    set Popup(expand) [$T column cget $Popup(column) -expand]
-	    set Popup(resize) [$T column cget $Popup(column) -resize]
-	    set Popup(squeeze) [$T column cget $Popup(column) -squeeze]
-	    set Popup(itemjustify) [$T column cget $Popup(column) -itemjustify]
-	    if {$Popup(itemjustify) eq ""} { set Popup(itemjustify) none }
-	    set Popup(justify) [$T column cget $Popup(column) -justify]
-	    set Popup(lock) [$T column cget $Popup(column) -lock]
-	    set Popup(treecolumn) [expr {[$T column id tree] eq $Popup(column)}]
-	    tk_popup $T.mHeader $X $Y
+    $T identify -array id $x $y
+    if {$id(where) ne ""} {
+	if {$id(where) eq "header"} {
+	    set H $id(header)
+	    set C $id(column)
+	    set Popup(header) $H
+	    set Popup(column) $C
+	    set Popup(arrow) [$T header cget $H $C -arrow]
+	    set Popup(arrow,side) [$T header cget $H $C -arrowside]
+	    set Popup(arrow,gravity) [$T header cget $H $C -arrowgravity]
+	    set Popup(button) [$T header cget $H $C -button]
+	    set Popup(header,justify) [$T header cget $H $C -justify]
+	    set Popup(header,visible) [$T header cget $H -visible]
+
+	    set Popup(header,drag,draw) [$T header dragcget $H -draw]
+	    set Popup(header,drag,enable) [$T header dragcget $H -enable]
+
+	    set Popup(column,expand,$C) [$T column cget $C -expand]
+	    set Popup(column,resize,$C) [$T column cget $C -resize]
+	    set Popup(column,squeeze,$C) [$T column cget $C -squeeze]
+	    set Popup(column,itemjustify,$C) [$T column cget $C -itemjustify]
+	    if {$Popup(column,itemjustify,$C) eq ""} { set Popup(column,itemjustify) none }
+	    set Popup(column,justify,$C) [$T column cget $C -justify]
+	    set Popup(column,lock,$C) [$T column cget $C -lock]
+	    set Popup(column,treecolumn,$C) [expr {[$T column id tree] eq $C}]
+	    $T.mColumn delete "Column"
+	    destroy $T.mColumn.mColumnX
+	    set m1 [MakeColumnSubmenu $T $C $T.mColumn "X"]
+	    $T.mColumn add cascade -label "Column" -menu $m1
+
+	    set m $T.mColumn.mHeaderColumn.mSpan
+	    $m delete 0 end
+	    if {[$T column compare $C == tail]} {
+		$m add checkbutton -label 1 -variable Popup(span)
+		set Popup(span) 1
+	    } else {
+		set lock [$T column cget $C -lock]
+		set last [expr {[$T column order "last lock $lock"] - [$T column order $C] + 1}]
+		for {set i 1} {$i <= $last} {incr i} {
+		    set break [expr {!(($i - 1) % 20)}]
+		    $m add radiobutton -label $i -command "$T header span $H $C $i" \
+			-variable Popup(span) -value $i -columnbreak $break
+		}
+		set Popup(span) [$T header span $H $C]
+	    }
+
+	    tk_popup $T.mColumn $X $Y
 	    return
 	}
     }
@@ -1236,22 +1399,18 @@ proc ShowPopup {T x y X Y} {
     set m $menu.mCollapse
     $m delete 0 end
     $m add command -label "All" -command {$Popup(T) item collapse all}
-    if {$id ne ""} {
-	if {[lindex $id 0] eq "item"} {
-	    set item [lindex $id 1]
-	    $m add command -label "Item $item" -command "$T item collapse $item"
-	    $m add command -label "Item $item (recurse)" -command "$T item collapse $item -recurse"
-	}
+    if {$id(where) eq "item"} {
+	set item $id(item)
+	$m add command -label "Item $item" -command "$T item collapse $item"
+	$m add command -label "Item $item (recurse)" -command "$T item collapse $item -recurse"
     }
     set m $menu.mExpand
     $m delete 0 end
     $m add command -label "All" -command {$Popup(T) item expand all}
-    if {$id ne ""} {
-	if {[lindex $id 0] eq "item"} {
-	    set item [lindex $id 1]
-	    $m add command -label "Item $item" -command "$T item expand $item"
-	    $m add command -label "Item $item (recurse)" -command "$T item expand $item -recurse"
-	}
+    if {$id(where) eq "item"} {
+	set item $id(item)
+	$m add command -label "Item $item" -command "$T item expand $item"
+	$m add command -label "Item $item (recurse)" -command "$T item expand $item -recurse"
     }
     foreach option {data display displaydelay enable span textlayout} {
 	set Popup(debug,$option) [$T debug cget -$option]
@@ -1280,39 +1439,57 @@ proc ShowPopup {T x y X Y} {
     set Popup(showrootbutton) [$T cget -showrootbutton]
     set Popup(showrootchildbuttons) [$T cget -showrootchildbuttons]
     set Popup(showrootlines) [$T cget -showrootlines]
-    set m $menu.mVisible
+
+    set m $menu.mColumns
+    eval destroy [winfo children $m]
     $m delete 0 end
     foreach C [$T column list] {
 	set break [expr {!([$T column order $C] % 20)}]
-	set Popup(visible,$C) [$T column cget $C -visible]
-	$m add checkbutton \
-	    -label "Column $C \"[$T column cget $C -text]\" \[[$T column cget $C -image]\]" \
-	    -variable Popup(visible,$C) \
-	    -command "$T column configure $C -visible \$Popup(visible,$C) ;
-		TreeCtrl::TryEvent $T DemoColumnVisibility {} {C $C}" \
-	    -columnbreak $break
+	set m1 [MakeColumnSubmenu $T $C $m]
+#	set m1 [menu $m.mColumn$C -postcommand [list PostColumnSubmenu $T $C $m]]
+	$m add cascade -menu $m1 -columnbreak $break \
+	    -label "Column $C \"[$T column cget $C -text]\" \[[$T column cget $C -image]\]"
+
+	set Popup(column,expand,$C) [$T column cget $C -expand]
+	set Popup(column,justify,$C) [$T column cget $C -justify]
+	set Popup(column,itemjustify,$C) [$T column cget $C -itemjustify]
+	if {$Popup(column,itemjustify,$C) eq ""} { set Popup(column,itemjustify,$C) none }
+	set Popup(column,lock,$C) [$T column cget $C -lock]
+	set Popup(column,squeeze,$C) [$T column cget $C -squeeze]
+	set Popup(column,visible,$C) [$T column cget $C -visible]
+	set Popup(treecolumn,$C) no
+	if {[$T column id tree] ne ""} {
+	    set Popup(treecolumn,$C) [$T column compare [$T column id tree] == $C]
+	}
+    }
+
+    set m $menu.mHeaders
+    eval destroy [winfo children $m]
+    $m delete 0 end
+    foreach H [$T header id all] {
+	set m1 [MakeHeaderSubmenu $T $H $m]
+	$m add cascade -menu $m1 -label "Header $H"
+	set Popup(header,visible,$H) [$T header cget $H -visible]
     }
 
     set m $menu.mItemWrap
     $m delete 0 end
     $m add command -label "All Off" -command {$Popup(T) item configure all -wrap off}
     $m add command -label "All On" -command {$Popup(T) item configure all -wrap on}
-    if {$id ne ""} {
-	if {[lindex $id 0] eq "item"} {
-	    set item [lindex $id 1]
-	    if {[$T item cget $item -wrap]} {
-		$m add command -label "Item $item Off" -command "$T item configure $item -wrap off"
-	    } else {
-		$m add command -label "Item $item On" -command "$T item configure $item -wrap on"
-	    }
+    if {$id(where) eq "item"} {
+	set item $id(item)
+	if {[$T item cget $item -wrap]} {
+	    $m add command -label "Item $item Off" -command "$T item configure $item -wrap off"
+	} else {
+	    $m add command -label "Item $item On" -command "$T item configure $item -wrap on"
 	}
     }
 
     set m $menu.mSpan
     $m delete 0 end
-    if {[llength $id] >= 4 && [lindex $id 2] eq "column"} {
-	set item [lindex $id 1]
-	set column [lindex $id 3]
+    if {$id(where) eq "item" && $id(column) ne ""} {
+	set item $id(item)
+	set column $id(column)
 	set lock [$T column cget $column -lock]
 	for {set i 1} {$i <= [$T column order "last lock $lock"] - [$T column order $column] + 1} {incr i} {
 	    set break [expr {!(($i - 1) % 20)}]
@@ -1360,10 +1537,10 @@ proc InitDemoList {} {
 	"Random $::RandomN Items, Button Images" DemoRandom2 random.tcl \
 	"Outlook Express (Folders)" DemoOutlookFolders outlook-folders.tcl \
 	"Outlook Express (Newsgroup)" DemoOutlookNewsgroup outlook-newgroup.tcl \
-	"Explorer (Details, WinXP)" DemoExplorerDetails explorer.tcl \
+	"Explorer (Details, Win98)" DemoExplorerDetails explorer.tcl \
 	"Explorer (Details, Win7)" DemoExplorerDetailsWin7 explorer.tcl \
 	"Explorer (List)" DemoExplorerList explorer.tcl \
-	"Explorer (Large icons, WinXP)" DemoExplorerLargeIcons explorer.tcl \
+	"Explorer (Large icons, Win98)" DemoExplorerLargeIcons explorer.tcl \
 	"Explorer (Large icons, Win7)" DemoExplorerLargeIconsWin7 explorer.tcl \
 	"Explorer (Small icons)" DemoExplorerSmallIcons explorer.tcl \
 	"Internet Options" DemoInternetOptions www-options.tcl \
@@ -1382,6 +1559,8 @@ proc InitDemoList {} {
 	"Gradients" DemoGradients gradients.tcl \
 	"Gradients II" DemoGradients2 gradients2.tcl \
 	"Gradients III" DemoGradients3 gradients3.tcl \
+	"Headers" DemoHeaders headers.tcl \
+	"Table" DemoTable table.tcl \
 	] {
 	set item [$t item create]
 	$t item lastchild root $item
@@ -1414,16 +1593,16 @@ proc TimerStop {{startTime ""}} {
     return [format "%.2g" [expr {($endTime - $startTime) / 1000000.0}]]
 }
 
-proc DemoSet {cmd file} {
+proc DemoSet {namespace file} {
     DemoClear
     TimerStart
-    uplevel #0 $cmd
+    uplevel #0 ${namespace}::Init [DemoList]
     dbwin "set list in [TimerStop] seconds\n"
     [DemoList] xview moveto 0
     [DemoList] yview moveto 0
     update
     DisplayStylesInList
-    ShowSource $file
+    SourceWindow::ShowSource $file
     catch {
 	if {[winfo ismapped .styleEditor]} {
 	    StyleEditor::SetListOfStyles
@@ -1633,9 +1812,12 @@ proc DemoClear {} {
 
     set T [DemoList]
 
-    # Clear the demo list
+    # Delete all the items (except the root item, it never gets deleted).
     $T item delete all
 
+    # Delete all the headers (except the first header, it never gets deleted).
+    $T header delete all
+
     # Clear all bindings on the demo list added by the previous demo.
     # The bindings are removed from the tag $T only. For those
     # bindings that should not be deleted we use the tag DontDelete.
@@ -1644,7 +1826,8 @@ proc DemoClear {} {
     $T notify unbind $T
 
     # Clear all run-time states
-    eval $T state undefine [$T state names]
+    eval $T header state undefine [$T header state names]
+    eval $T item state undefine [$T item state names]
 
     # Clear the styles-in-item list
     .f3.t item delete all
@@ -1660,7 +1843,7 @@ proc DemoClear {} {
 
     # Delete -window windows
     foreach child [winfo children $T] {
-	if {[string equal $child $T.mTree] || [string equal $child $T.mHeader]} continue
+	if {[string equal $child $T.mTree] || [string equal $child $T.mColumn]} continue
 	destroy $child
     }
 
@@ -1673,6 +1856,13 @@ proc DemoClear {} {
     $T item configure root -button no -wrap no
     $T item expand root
 
+    # Restore header defaults
+    foreach spec [$T header configure 0] {
+	if {[llength $spec] == 2} continue
+	lassign $spec name x y default current
+	$T header configure all $name $default
+    }
+
     # Restore some happy defaults to the demo list
     foreach spec [$T configure] {
 	if {[llength $spec] == 2} continue
@@ -1682,6 +1872,9 @@ proc DemoClear {} {
     $T configure -background white
     $T configure -borderwidth [expr {$::tileFull ? 0 : 6}]
     $T configure -font DemoFont
+    if {[Platform unix]} {
+	$T configure -headerfont DemoFont
+    }
     $T configure -highlightthickness [expr {$::tileFull ? 0 : 3}]
     $T configure -relief ridge
 
@@ -1700,7 +1893,12 @@ proc DemoClear {} {
 
     # Enable drag-and-drop column reordering. This also requires the
     # <ColumnDrag> event be installed.
-    $T column dragconfigure -enable yes
+    $T header dragconfigure -enable yes
+    $T header dragconfigure all -enable yes -draw yes
+
+    # Re-active the column drag-and-drop binding in case the previous demo
+    # deactivated it.
+    $T notify configure DontDelete <ColumnDrag-receive> -active yes
 
     # Restore default bindings to the demo list
     bindtags $T [list $T TreeCtrl [winfo toplevel $T] all DisplayStylesInItemBindTag]
@@ -1722,6 +1920,8 @@ proc DemoPictureCatalog {} {
 	-selectmode multiple -orient horizontal -wrap window \
 	-yscrollincrement 50 -showheader no
 
+    $T column create
+
     $T element create elemTxt text -fill {SystemHighlightText {selected focus}}
     $T element create elemSelTxt rect -fill {SystemHighlight {selected focus}}
     $T element create elemSelImg rect -outline {SystemHighlight {selected focus}} \
@@ -1755,8 +1955,10 @@ proc DemoPictureCatalog2 {} {
 	-selectmode multiple -orient horizontal -wrap window \
 	-yscrollincrement 50 -showheader no
 
+    $T column create
+
     $T element create elemTxt text -fill {SystemHighlightText {selected focus}} \
-	-justify left -wrap word -lines 2
+	-justify left -wrap word -lines 3
     $T element create elemSelTxt rect -fill {SystemHighlight {selected focus}}
     $T element create elemSelImg rect -outline {SystemHighlight {selected focus}} \
 	-outlinewidth 4
@@ -1909,32 +2111,34 @@ proc CursorWindow {} {
 # A little screen magnifier
 if {[llength [info commands loupe]]} {
 
-    set Loupe(zoom) 2
-    set Loupe(x) 0
-    set Loupe(y) 0
-    set Loupe(auto) 1
-    set Loupe(afterId) ""
+    namespace eval LoupeWindow {
+	variable Priv
+	set Priv(zoom) 2
+	set Priv(x) 0
+	set Priv(y) 0
+	set Priv(auto) 1
+	set Priv(afterId) ""
+	set Priv(image) ::LoupeWindow::Image
+	set Priv(delay) 500
+    }
 
-    proc LoupeAfter {} {
-
-	global Loupe
+    proc LoupeWindow::After {} {
+	variable Priv
 	set x [winfo pointerx .]
 	set y [winfo pointery .]
-	if {$Loupe(auto) || ($Loupe(x) != $x) || ($Loupe(y) != $y)} {
-	    set w [image width $Loupe(image)]
-	    set h [image height $Loupe(image)]
-	    loupe $Loupe(image) $x $y $w $h $::Loupe(zoom)
-	    set Loupe(x) $x
-	    set Loupe(y) $y
+	if {$Priv(auto) || ($Priv(x) != $x) || ($Priv(y) != $y)} {
+	    set w [image width $Priv(image)]
+	    set h [image height $Priv(image)]
+	    loupe $Priv(image) $x $y $w $h $Priv(zoom)
+	    set Priv(x) $x
+	    set Priv(y) $y
 	}
-	set Loupe(afterId) [after $Loupe(delay) LoupeAfter]
+	set Priv(afterId) [after $Priv(delay) LoupeWindow::After]
 	return
     }
 
-    proc MakeLoupeWindow {} {
-
-	global Loupe
-
+    proc LoupeWindow::Init {} {
+	variable Priv
 	set w [toplevel .loupe]
 	wm title $w "TreeCtrl Magnifier"
 	wm withdraw $w
@@ -1943,50 +2147,51 @@ if {[llength [info commands loupe]]} {
 	} else {
 	    wm geometry $w -0+0
 	}
-	image create photo ImageLoupe -width 280 -height 150
-	pack [label $w.label -image ImageLoupe -borderwidth 1 -relief sunken] \
+	image create photo $Priv(image) -width 280 -height 150
+	pack [label $w.label -image $Priv(image) -borderwidth 1 -relief sunken] \
 	    -expand yes -fill both
 
 	set f [frame $w.zoom -borderwidth 0]
-	radiobutton $f.r1 -text "1x" -variable ::Loupe(zoom) -value 1
-	radiobutton $f.r2 -text "2x" -variable ::Loupe(zoom) -value 2
-	radiobutton $f.r4 -text "4x" -variable ::Loupe(zoom) -value 4
-	radiobutton $f.r8 -text "8x" -variable ::Loupe(zoom) -value 8
+	radiobutton $f.r1 -text "1x" -variable ::LoupeWindow::Priv(zoom) -value 1
+	radiobutton $f.r2 -text "2x" -variable ::LoupeWindow::Priv(zoom) -value 2
+	radiobutton $f.r4 -text "4x" -variable ::LoupeWindow::Priv(zoom) -value 4
+	radiobutton $f.r8 -text "8x" -variable ::LoupeWindow::Priv(zoom) -value 8
 	pack $f.r1 $f.r2 $f.r4 $f.r8 -side left
 	pack $f -side bottom -anchor center
 
 	# Resize the image with the window
 	bind LoupeWindow <Configure> {
-	    set w [expr {%w - 2}]
-	    set h [expr {%h - 2}]
-	    if {$w != [$Loupe(image) cget -width] ||
-		$h != [$Loupe(image) cget -height]} {
-		$Loupe(image) configure -width $w -height $h
-		loupe $Loupe(image) $Loupe(x) $Loupe(y) $w $h $Loupe(zoom)
-	    }
+	    LoupeWindow::ResizeImage %w %h
 	}
 	bindtags $w.label [concat [bindtags .loupe] LoupeWindow]
 
-	wm protocol $w WM_DELETE_WINDOW "ToggleLoupeWindow"
-
-	set Loupe(image) ImageLoupe
-	set Loupe(delay) 500
+	wm protocol $w WM_DELETE_WINDOW "LoupeWindow::ToggleWindowVisibility"
 	return
     }
 
-    proc ToggleLoupeWindow {} {
-
-	global Loupe
+    proc LoupeWindow::ResizeImage {w h} {
+	variable Priv
+	set w [expr {$w - 2}]
+	set h [expr {$h - 2}]
+	if {$w != [$Priv(image) cget -width] ||
+	    $h != [$Priv(image) cget -height]} {
+	    $Priv(image) configure -width $w -height $h
+	    loupe $Priv(image) $Priv(x) $Priv(y) $w $h $Priv(zoom)
+	}
+	return
+    }
 
+    proc LoupeWindow::ToggleWindowVisibility {} {
+	variable Priv
 	set w .loupe
 	if {![winfo exists $w]} {
-	    MakeLoupeWindow
+	    LoupeWindow::Init
 	}
 	if {[winfo ismapped $w]} {
-	    after cancel $Loupe(afterId)
+	    after cancel $Priv(afterId)
 	    wm withdraw $w
 	} else {
-	    LoupeAfter
+	    After
 	    wm deiconify $w
 	    raise $w
 	}
@@ -2005,3 +2210,4 @@ proc RandomPerfTest {} {
     puts [time {[DemoList] colu conf 0 -width 160 ; update}]
     return
 }
+
