@@ -1,4 +1,4 @@
-# RCS: @(#) $Id: explorer.tcl,v 1.41 2010/12/05 04:12:23 treectrl Exp $
+# Copyright (c) 2002-2011 Tim Baker
 
 set Dir [file dirname [file dirname [info script]]]
 
@@ -19,11 +19,21 @@ if {[info commands ::tk::mac::iconBitmap] ne {}} {
     set macBitmap 1
 }
 
-proc DemoExplorerAux {scriptDir scriptFile {scriptFollowup ""}} {
-    global Explorer
-    global Dir
+namespace eval DemoExplorer {}
 
-    set T [DemoList]
+# DemoExplorer::SetList
+#
+# Gets sorted lists of directory and file names in the ::Dir directory and
+# calls a script to add items to the demo list.
+#
+# Arguments:
+# scriptDir		Script to eval to add directories to the demo list.
+# scriptFile		Script to eval to add files to the demo list.
+# scriptFollowup	Script to eval after the first two.
+
+proc DemoExplorer::SetList {T scriptDir scriptFile {scriptFollowup ""}} {
+    variable Priv
+    global Dir
 
     TimerStart
     set globDirs [glob -nocomplain -types d -dir $Dir *]
@@ -64,46 +74,66 @@ proc DemoExplorerAux {scriptDir scriptFile {scriptFollowup ""}} {
     set af $secondsAddFiles
     puts "dirs([llength $globDirs]) glob/sort/add $gd/$sd/$ad\nfiles([llength $globFiles]) glob/sort/add $gf/$sf/$af"
 
+    # Accessing a private variable in the filelist-bindings.tcl library script.
+    # Most of the code in that file should have been a part of this demo.
     set ::TreeCtrl::Priv(DirCnt,$T) [llength $globDirs]
 
     eval $scriptFollowup
 
     # Double-clicking a directory displays its contents.
-    set Explorer(scriptDir) $scriptDir
-    set Explorer(scriptFile) $scriptFile
-    set Explorer(scriptFollowup) $scriptFollowup
+    set Priv(scriptDir) $scriptDir
+    set Priv(scriptFile) $scriptFile
+    set Priv(scriptFollowup) $scriptFollowup
 
     return
 }
 
-proc DemoExplorerBindings {{win7 0}} {
+# DemoExplorer::SetBindings
+#
+# Sets some bindings on the demo list.
+#
+# Arguments:
+# T			The demo list.
+# win7			Boolean, true if Windows 7 behavior is desired.
+
+proc DemoExplorer::SetBindings {T {win7 0}} {
+
+    variable Priv
 
     # Double-clicking a directory displays its contents.
     bind DemoExplorer <Double-ButtonPress-1> {
-	ExplorerDoubleButton1 %W %x %y
+	DemoExplorer::DoubleButton1 %W %x %y
     }
 
-    TreeCtrl::FileListEmulateWin7 [DemoList] $win7
+    TreeCtrl::FileListEmulateWin7 $T $win7
 
     if {$win7} {
-	set ::Explorer(prev) ""
+	set Priv(prev) ""
+	$T notify bind $T <ItemDelete> {
+	    if {[lsearch -exact %i $DemoExplorer::Priv(prev)] != -1} {
+		set DemoExplorer::Priv(prev) ""
+	    }
+	}
 	bind DemoExplorerWin7 <Motion> {
-	    ExplorerMotion %W %x %y
+	    DemoExplorer::Motion %W %x %y
 	}
 	bind DemoExplorerWin7 <Leave> {
-	    ExplorerMotion %W -1 -1
+	    DemoExplorer::Motion %W -1 -1
 	}
     }
+
+    return
 }
 
 #
 # Demo: explorer files
 #
-proc DemoExplorerDetails {} {
+namespace eval DemoExplorerDetails {
+    proc Init {T} { DemoExplorer::InitDetails $T }
+}
+proc DemoExplorer::InitDetails {T} {
 
-    global Explorer
-
-    set T [DemoList]
+    variable Priv
 
     set height [font metrics [$T cget -font] -linespace]
     if {$height < 18} {
@@ -201,7 +231,7 @@ proc DemoExplorerDetails {} {
     }
 
     # During editing, hide the text and selection-rectangle elements.
-    $T state define edit
+    $T item state define edit
     $T style layout styName txtName -draw {no edit}
     $T style layout styName elemRectSel -draw {no edit}
     $T notify bind $T <Edit-begin> {
@@ -280,22 +310,32 @@ proc DemoExplorerDetails {} {
 	$T item lastchild root $item
     }
 
-    DemoExplorerAux $scriptDir $scriptFile
-    DemoExplorerBindings
+    SetList $T $scriptDir $scriptFile
+    SetBindings $T
 
-    set ::Explorer(sortColumn) name
-    set ::Explorer(sortColor) #F7F7F7
-    $T notify bind $T <Header-invoke> { ExplorerHeaderInvoke %T %C }
+    set Priv(sortColumn) name
+    set Priv(sortColor) #F7F7F7
+    $T notify bind $T <Header-invoke> { DemoExplorer::HeaderInvoke %T %C }
 
     bindtags $T [list $T DemoExplorer TreeCtrlFileList TreeCtrl [winfo toplevel $T] all]
 
     return
 }
 
-proc ExplorerHeaderInvoke {T C} {
-    global Explorer
-    if {[$T column compare $C == $Explorer(sortColumn)]} {
-	if {[$T column cget $Explorer(sortColumn) -arrow] eq "down"} {
+# DemoExplorer::HeaderInvoke
+#
+# This procedure is called to handle the <Header-invoke> event generated by
+# the treectrl.tcl library script.  Items in the demo list are sorted
+# according to the column involved.
+#
+# Arguments:
+# T			The demo list.
+# C			The column whose header was clicked.
+
+proc DemoExplorer::HeaderInvoke {T C} {
+    variable Priv
+    if {[$T column compare $C == $Priv(sortColumn)]} {
+	if {[$T column cget $Priv(sortColumn) -arrow] eq "down"} {
 	    set order -increasing
 	    set arrow up
 	} else {
@@ -303,17 +343,17 @@ proc ExplorerHeaderInvoke {T C} {
 	    set arrow down
 	}
     } else {
-	if {[$T column cget $Explorer(sortColumn) -arrow] eq "down"} {
+	if {[$T column cget $Priv(sortColumn) -arrow] eq "down"} {
 	    set order -decreasing
 	    set arrow down
 	} else {
 	    set order -increasing
 	    set arrow up
 	}
-	$T column configure $Explorer(sortColumn) -arrow none -itembackground {}
-	set Explorer(sortColumn) $C
+	$T column configure $Priv(sortColumn) -arrow none -itembackground {}
+	set Priv(sortColumn) $C
     }
-    $T column configure $C -arrow $arrow -itembackground $Explorer(sortColor)
+    $T column configure $C -arrow $arrow -itembackground $Priv(sortColor)
     set dirCount $::TreeCtrl::Priv(DirCnt,$T)
     set fileCount [expr {[$T item count] - 1 - $dirCount}]
     set lastDir [expr {$dirCount - 1}]
@@ -345,12 +385,14 @@ proc ExplorerHeaderInvoke {T C} {
 	    }
 	}
     }
+
     return
 }
 
-proc DemoExplorerLargeIcons {} {
-
-    set T [DemoList]
+namespace eval DemoExplorerLargeIcons {
+    proc Init {T} { DemoExplorer::InitLargeIcons $T }
+}
+proc DemoExplorer::InitLargeIcons {T} {
 
     # Item height is 32 for icon, 4 padding, 3 lines of text
     set itemHeight [expr {32 + 4 + [font metrics [$T cget -font] -linespace] * 3}]
@@ -422,7 +464,7 @@ proc DemoExplorerLargeIcons {} {
     }
 
     # During editing, hide the text and selection-rectangle elements.
-    $T state define edit
+    $T item state define edit
     $T style layout STYLE elemTxt -draw {no edit}
     $T style layout STYLE elemSel -draw {no edit}
     $T notify bind $T <Edit-begin> {
@@ -491,8 +533,8 @@ proc DemoExplorerLargeIcons {} {
 	$T item lastchild root $item
     }
 
-    DemoExplorerAux $scriptDir $scriptFile
-    DemoExplorerBindings
+    SetList $T $scriptDir $scriptFile
+    SetBindings $T
 
     $T activate [$T item id "root firstchild"]
 
@@ -513,9 +555,11 @@ proc DemoExplorerLargeIcons {} {
 
 # Tree is horizontal, wrapping occurs at right edge of window, each item
 # is as wide as the smallest needed multiple of 110 pixels
-proc DemoExplorerSmallIcons {} {
-    set T [DemoList]
-    DemoExplorerList
+namespace eval DemoExplorerSmallIcons {
+    proc Init {T} { DemoExplorer::InitSmallIcons $T }
+}
+proc DemoExplorer::InitSmallIcons {T} {
+    InitList $T
     $T configure -orient horizontal \
 	-itemwidthmultiple 110 -itemwidthequal no
     return
@@ -523,9 +567,10 @@ proc DemoExplorerSmallIcons {} {
 
 # Tree is vertical, wrapping occurs at bottom of window, each range has the
 # same width (as wide as the longest item), xscrollincrement is by range
-proc DemoExplorerList {} {
-
-    set T [DemoList]
+namespace eval DemoExplorerList {
+    proc Init {T} { DemoExplorer::InitList $T }
+}
+proc DemoExplorer::InitList {T} {
 
     set height [font metrics [$T cget -font] -linespace]
     if {$height < 18} {
@@ -599,7 +644,7 @@ proc DemoExplorerList {} {
     }
 
     # During editing, hide the text and selection-rectangle elements.
-    $T state define edit
+    $T item state define edit
     $T style layout STYLE elemTxt -draw {no edit}
     $T style layout STYLE elemSel -draw {no edit}
     $T notify bind $T <Edit-begin> {
@@ -668,8 +713,8 @@ proc DemoExplorerList {} {
 	$T item lastchild root $item
     }
 
-    DemoExplorerAux $scriptDir $scriptFile
-    DemoExplorerBindings
+    SetList $T $scriptDir $scriptFile
+    SetBindings $T
 
     $T activate [$T item firstchild root]
 
@@ -678,17 +723,27 @@ proc DemoExplorerList {} {
     return
 }
 
-proc ExplorerDoubleButton1 {T x y} {
-    global Explorer
+# DemoExplorer::DoubleButton1
+#
+# Handle the <Double-ButtonPress-1> event in the demo list.  If a directory
+# item is double-clicked, display that directory.
+#
+# Arguments:
+# T			The demo list.
+# x			Widget x coordinate.
+# y			Widget y coordinate.
+
+proc DemoExplorer::DoubleButton1 {T x y} {
+    variable Priv
     global Dir
-    set id [$T identify $x $y]
+    $T identify -array id $x $y
     set sensitive [TreeCtrl::IsSensitive $T $x $y]
     if {[TreeCtrl::FileListEmulateWin7 $T] && [TreeCtrl::IsSensitiveMarquee $T $x $y]} {
 	set sensitive 1
     }
     if {$sensitive} {
-	set item [lindex $id 1]
-	set column [lindex $id 3]
+	set item $id(item)
+	set column $id(column)
 	if {[$T item tag expr $item directory]} {
 	    set name [$T item text $item {tag C0}]
 	    if {$name eq ".."} {
@@ -697,7 +752,7 @@ proc ExplorerDoubleButton1 {T x y} {
 		set Dir [file join $Dir $name]
 	    }
 	    $T item delete all
-	    DemoExplorerAux $Explorer(scriptDir) $Explorer(scriptFile) $Explorer(scriptFollowup)
+	    SetList $T $Priv(scriptDir) $Priv(scriptFile) $Priv(scriptFollowup)
 	    if {![TreeCtrl::FileListEmulateWin7 $T]} {
 		$T activate "root firstchild"
 	    }
@@ -708,8 +763,12 @@ proc ExplorerDoubleButton1 {T x y} {
     return
 }
 
+# DemoExplorer::DragStyleInit
+#
+# NOT USED.
 # Experimental code to create a style that is used with drag-and-drop
-proc DragStyleInit {} {
+
+proc DemoExplorer::DragStyleInit {} {
 
     set T [DemoList]
 
@@ -748,9 +807,15 @@ proc DragStyleInit {} {
     return
 }
 
+# DemoExplorer::ConfigTransparentMarquee
+#
 # Configure the marquee for a modern transparent selection rectangle
 # where transparent gradients are supported.
-proc ConfigTransparentMarquee {T} {
+#
+# Arguments:
+# T			The demo list.
+
+proc DemoExplorer::ConfigTransparentMarquee {T} {
     if {!$::NativeGradients} return
     if {![$T gradient native]} return
     if {[winfo depth $T] < 15} return
@@ -766,11 +831,12 @@ proc ConfigTransparentMarquee {T} {
 #
 # Demo: explorer files with Windows-7-like gradients
 #
-proc DemoExplorerDetailsWin7 {} {
+namespace eval DemoExplorerDetailsWin7 {
+    proc Init {T} { DemoExplorer::InitDetailsWin7 $T }
+}
+proc DemoExplorer::InitDetailsWin7 {T} {
 
-    global Explorer
-
-    set T [DemoList]
+    variable Priv
 
     set height [font metrics [$T cget -font] -linespace]
     if {$height < 16} {
@@ -833,10 +899,10 @@ proc DemoExplorerDetailsWin7 {} {
     $T element create txtSize text -datatype integer -format "%dKB" -lines 1 -fill #6d6d6d
     $T element create txtDate text -datatype time -format "%d/%m/%y %I:%M %p" -lines 1 -fill #6d6d6d
 
-    $T state define mouseover
-    $T state define openW
-    $T state define openE
-    $T state define openWE
+    $T item state define mouseover
+    $T item state define openW
+    $T item state define openE
+    $T item state define openWE
 
     $T element create elemRectGradient rect \
 	-fill [list	G_selected_active {selected mouseover} \
@@ -918,7 +984,7 @@ proc DemoExplorerDetailsWin7 {} {
     }
 
     # During editing, hide the text
-    $T state define edit
+    $T item state define edit
     $T style layout styName txtName -draw {no edit}
     $T notify bind $T <Edit-begin> {
 	%T item state set %I ~edit
@@ -996,35 +1062,43 @@ proc DemoExplorerDetailsWin7 {} {
     }
 
     set scriptFollowup {
-	DemoExplorerDetailsWin7_FixItemStyles $T
+	DemoExplorer::DetailsWin7_FixItemStyles $T
     }
 
-    DemoExplorerAux $scriptDir $scriptFile $scriptFollowup
-    DemoExplorerBindings true
+    SetList $T $scriptDir $scriptFile $scriptFollowup
+    SetBindings $T true
 
     ConfigTransparentMarquee $T
 
     # Fix the display when a column is dragged
     $T notify bind $T <ColumnDrag-receive> {
 	%T column move %C %b
-	DemoExplorerDetailsWin7_FixItemStyles %T
+	DemoExplorer::DetailsWin7_FixItemStyles %T
     }
 
     # Fix the display when a column's visibility changes
     $T notify bind $T <DemoColumnVisibility> {
-	DemoExplorerDetailsWin7_FixItemStyles %T
+	DemoExplorer::DetailsWin7_FixItemStyles %T
     }
 
-    set ::Explorer(sortColumn) name
-    set ::Explorer(sortColor) ""
-    $T notify bind $T <Header-invoke> { ExplorerHeaderInvoke %T %C }
+    set Priv(sortColumn) name
+    set Priv(sortColor) ""
+    $T notify bind $T <Header-invoke> { DemoExplorer::HeaderInvoke %T %C }
 
     bindtags $T [list $T DemoExplorerWin7 DemoExplorer TreeCtrlFileList TreeCtrl [winfo toplevel $T] all]
 
     return
 }
 
-proc DemoExplorerDetailsWin7_FixItemStyles {T} {
+# DemoExplorer::DetailsWin7_FixItemStyles
+#
+# Configures item states and style layouts so that selection rectangles
+# appear to span multiple columns.
+#
+# Arguments:
+# T			The demo list.
+
+proc DemoExplorer::DetailsWin7_FixItemStyles {T} {
     foreach C [$T column id "visible !tail"] {
 	if {[$T column compare $C == "first visible"]} {
 	    set padx {2 0}
@@ -1055,11 +1129,13 @@ proc DemoExplorerDetailsWin7_FixItemStyles {T} {
 	$T item state forcolumn all $C [list !openW !openE !openWE $state]
 	$T style layout $style elemRectGradient -padx $padx
     }
+    return
 }
 
-proc DemoExplorerLargeIconsWin7 {} {
-
-    set T [DemoList]
+namespace eval DemoExplorerLargeIconsWin7 {
+    proc Init {T} { DemoExplorer::InitLargeIconsWin7 $T }
+}
+proc DemoExplorer::InitLargeIconsWin7 {T} {
 
     # Item height is 2 + 32 for icon + 4 pad + 3 lines of text + 3
     set fontHeight [font metrics [$T cget -font] -linespace]
@@ -1098,10 +1174,10 @@ proc DemoExplorerLargeIconsWin7 {} {
     # Create elements
     #
 
-    $T state define mouseover
-    $T state define openW
-    $T state define openE
-    $T state define openWE
+    $T item state define mouseover
+    $T item state define openW
+    $T item state define openE
+    $T item state define openWE
 
     $T element create elemRectGradient rect \
 	-fill [list	G_selected_active {selected mouseover} \
@@ -1166,7 +1242,7 @@ proc DemoExplorerLargeIconsWin7 {} {
     }
 
     # During editing, hide the text and selection-rectangle elements.
-    $T state define edit
+    $T item state define edit
     $T style layout STYLE elemTxt -draw {no edit}
     $T notify bind $T <Edit-begin> {
 	%T item state set %I ~edit
@@ -1234,8 +1310,8 @@ proc DemoExplorerLargeIconsWin7 {} {
 	$T item lastchild root $item
     }
 
-    DemoExplorerAux $scriptDir $scriptFile
-    DemoExplorerBindings true
+    SetList $T $scriptDir $scriptFile
+    SetBindings $T true
 
     ConfigTransparentMarquee $T
 
@@ -1256,36 +1332,47 @@ proc DemoExplorerLargeIconsWin7 {} {
     return
 }
 
-# Win7: Handle <Motion> and <Leave> events.
-proc ExplorerMotion {w x y} {
-    global Explorer
+# DemoExplorer::Motion
+#
+# Handle <Motion> and <Leave> events.  In the Windows 7 demos, this highlights
+# the item under the mouse pointer.
+#
+# Arguments:
+# T			The demo list.
+# x			Widget x coordinate.
+# y			Widget y coordinate.
+
+proc DemoExplorer::Motion {T x y} {
+    variable Priv
 
     # Check for Win7 'mouseover' state
-    if {[lsearch -exact [$w state names] mouseover] == -1} return
+    if {[lsearch -exact [$T item state names] mouseover] == -1} return
 
     if {[info exists TreeCtrl::Priv(buttonMode)] && $TreeCtrl::Priv(buttonMode) ne ""} {
 	set x [set y -1]
     }
 
-    set id [$w identify $x $y]
-    if {$id eq ""} {
-    } elseif {[lindex $id 0] eq "header"} {
-    } elseif {[lindex $id 0] eq "item" && [llength $id] > 4} {
-	set item [lindex $id 1]
-	if {$item ne $Explorer(prev)} {
-	    if {$Explorer(prev) ne ""} {
-		$w item state set $Explorer(prev) !mouseover
+    $T identify -array id $x $y
+    if {$id(where) eq ""} {
+	# nothing
+    } elseif {$id(where) eq "header"} {
+	# nothing
+    } elseif {$id(where) eq "item" && $id(element) ne ""} {
+	set item $id(item)
+	if {$item ne $Priv(prev)} {
+	    if {$Priv(prev) ne ""} {
+		$T item state set $Priv(prev) !mouseover
 	    }
-	    $w item state set $item mouseover
-	    set Explorer(prev) $item
+	    $T item state set $item mouseover
+	    set Priv(prev) $item
 	}
 	return
     }
-    if {$Explorer(prev) ne ""} {
-	if {[$w item id $Explorer(prev)] ne ""} {
-	    $w item state set $Explorer(prev) !mouseover
+    if {$Priv(prev) ne ""} {
+	if {[$T item id $Priv(prev)] ne ""} {
+	    $T item state set $Priv(prev) !mouseover
 	}
-	set Explorer(prev) ""
+	set Priv(prev) ""
     }
     return
 }
