@@ -1,4 +1,4 @@
-# RCS: @(#) $Id: style-editor.tcl,v 1.21 2010/12/05 04:12:23 treectrl Exp $
+# Copyright (c) 2005-2011 Tim Baker
 
 namespace eval StyleEditor {
     variable Info
@@ -83,7 +83,7 @@ proc StyleEditor::Init {Tdemo} {
     #
 
     set fCanvas [frame $fRight.fCanvas -borderwidth 1 -relief sunken]
-    set canvas [canvas $fCanvas.canvas -background white \
+    set canvas [canvas $fCanvas.canvas -background white -height 300 \
 	-scrollregion {0 0 0 0} -borderwidth 0 -highlightthickness 0 \
 	-xscrollcommand [list sbset $fCanvas.xscroll] \
 	-yscrollcommand [list sbset $fCanvas.yscroll]]
@@ -101,6 +101,28 @@ proc StyleEditor::Init {Tdemo} {
 
     Info canvas $canvas
 
+    Info selectedStyle ""
+    Info selectedElement ""
+
+    #
+    # Create some scale controls to test expansion and squeezing
+    #
+
+    $::scaleCmd $canvas.scaleX \
+	-orient horizontal -length 300 -from 5 -to 150 \
+	-command StyleEditor::ScaleX
+    if {!$::tile} {$canvas.scaleX configure -showvalue no}
+    place $canvas.scaleX -rely 1.0 -anchor sw
+    $::scaleCmd $canvas.scaleY \
+	-orient vertical -length 300 -from 5 -to 150 \
+	-command StyleEditor::ScaleY
+    if {!$::tile} {$canvas.scaleY configure -showvalue no}
+    place $canvas.scaleY -relx 1.0 -anchor ne
+    $canvas.scaleX set 150
+    $canvas.scaleY set 150
+    Info scaleX,widget $canvas.scaleX
+    Info scaleY,widget $canvas.scaleY
+
     # Create a new treectrl to copy the states/style/elements into, so I don't
     # have to worry about column width or item visibility in the demo list
     set T [treectrl $canvas.t]
@@ -122,9 +144,6 @@ proc StyleEditor::Init {Tdemo} {
     grid columnconfigure $w 0 -weight 1
     grid $w.pwH -row 0 -column 0 -sticky news
 
-    Info selectedStyle ""
-    Info selectedElement ""
-
     wm protocol $w WM_DELETE_WINDOW "ToggleStyleEditorWindow"
     if {[Platform macosx macintosh]} {
 	wm geometry $w +6+30
@@ -187,6 +206,8 @@ proc StyleEditor::SelectStyle {} {
     SetListOfElements $style
 
     Info -orient [$Tdemo style cget $style -orient]
+    [Info scaleX,widget] set 150
+    [Info scaleY,widget] set 150
 
     StyleToCanvas 1
 
@@ -259,7 +280,7 @@ proc StyleEditor::SetPropertyList {} {
 
     # Create elements and styles the first time this is called
     if {[llength [$T style names]] == 0} {
-	$T state define header
+	$T item state define header
 
 	$T element create e1 text \
 	    -fill [list white header $::SystemHighlightText selected] \
@@ -280,7 +301,7 @@ proc StyleEditor::SetPropertyList {} {
 	set S [$T style create s1]
 	$T style elements $S {e2 e1}
 	$T style layout $S e2 -detach yes -indent no -iexpand xy
-	$T style layout $S e1 -expand ns -padx {4 0}
+	$T style layout $S e1 -expand ns -padx 4
 
 	set S [$T style create sWindow]
 	$T style elements $S {e2 eWindow}
@@ -290,7 +311,7 @@ proc StyleEditor::SetPropertyList {} {
 	set S [$T style create sHeader]
 	$T style elements $S {e2 e1}
 	$T style layout $S e2 -detach yes -iexpand xy
-	$T style layout $S e1 -expand ns -padx {4 0}
+	$T style layout $S e1 -expand ns -padx 4
 
 	Info editor,pad [MakePadEditor $T]
 	Info editor,expand [MakeExpandEditor $T]
@@ -323,6 +344,8 @@ proc StyleEditor::SetPropertyList {} {
 	"Draw and Visible" ""
 	"" -draw
 	"" -visible
+	"Center" ""
+	"" -center
 	"Detach" ""
 	"" -detach
 	"" -indent
@@ -375,7 +398,7 @@ proc StyleEditor::SelectProperty {select deselect} {
     set style [Info selectedStyle]
     set element [Info selectedElement]
 
-    if {[llength $deselect] && ($element ne "")} { 
+    if {[llength $deselect] && ($element ne "")} {
 	set I [lindex $deselect 0]
 	if {[$T item tag expr $I !header]} {
 	    set option [$T item text $I property]
@@ -443,6 +466,7 @@ proc StyleEditor::SelectProperty {select deselect} {
 	    $T item element configure $I value eWindow -window [Info editor,boolean]
 	    Info -boolean [$Tdemo style layout $style $element $option]
 	}
+	-center -
 	-squeeze {
 	    $T item element configure $I value eWindow -window [Info editor,squeeze]
 	    set value [$Tdemo style layout $style $element $option]
@@ -667,7 +691,7 @@ proc StyleEditor::Sync_squeeze {} {
 	    append value $flag
 	}
     }
-    $Tdemo style layout $style $element -squeeze $value
+    $Tdemo style layout $style $element $option $value
     StyleToCanvas
     return
 }
@@ -733,15 +757,22 @@ proc StyleEditor::StyleToCanvas {{scroll 0}} {
     set T $canvas.t
 
     $T configure -itemheight 0
+    $T item configure root -height 0
+    $T header configure first -height 0
     $T column configure 0 -width {}
-    eval $T state undefine [$T state names]
+
+    # Get the state domain, either "item" or "header".
+    # It is used as the command name as well as the -statedomain value.
+    set domain [$Tdemo style cget $style -statedomain]
+
+    eval $T $domain state undefine [$T $domain state names]
     eval $T style delete [$T style names]
     eval $T element delete [$T element names]
     eval $T gradient delete [$T gradient names]
 
     # Copy states
-    foreach state [$Tdemo state names] {
-	$T state define $state
+    foreach state [$Tdemo $domain state names] {
+	$T $domain state define $state
     }
 
     # Copy gradients (name only)
@@ -751,7 +782,8 @@ proc StyleEditor::StyleToCanvas {{scroll 0}} {
 
     # Copy elements
     foreach E [$Tdemo style elements $style] {
-	$T element create $E [$Tdemo element type $E]
+	$T element create $E [$Tdemo element type $E] \
+	    -statedomain $domain
 	foreach list [$Tdemo element configure $E] {
 	    lassign $list name x y default current
 	    $T element configure $E $name $current
@@ -759,27 +791,21 @@ proc StyleEditor::StyleToCanvas {{scroll 0}} {
     }
 
     # Copy style
-    $T style create $style -orient [$Tdemo style cget $style -orient]
+    $T style create $style -orient [$Tdemo style cget $style -orient] \
+	-statedomain $domain
     $T style elements $style [$Tdemo style elements $style]
     foreach E [$T style elements $style] {
 	eval $T style layout $style $E [$Tdemo style layout $style $E]
 	#$T style layout $style $E -visible {}
     }
 
-    # Find a selected item using the style to copy element config info from
-    set match ""
-    foreach I [$Tdemo selection get] {
-	foreach S [$Tdemo item style set $I] C [$Tdemo column list] {
-	    if {$S eq $style} {
-		set match $I
-		break
-	    }
-	}
-	if {$match ne ""} break
+    if {$domain eq "header"} {
+	set match ""
     }
-    # No selected item uses the current style, look for an unselected item
-    if {$match eq ""} {
-	foreach I [$Tdemo item range first last] {
+    if {$domain eq "item"} {
+	# Find a selected item using the style to copy element config info from
+	set match ""
+	foreach I [$Tdemo selection get] {
 	    foreach S [$Tdemo item style set $I] C [$Tdemo column list] {
 		if {$S eq $style} {
 		    set match $I
@@ -788,6 +814,18 @@ proc StyleEditor::StyleToCanvas {{scroll 0}} {
 	    }
 	    if {$match ne ""} break
 	}
+	# No selected item uses the current style, look for an unselected item
+	if {$match eq ""} {
+	    foreach I [$Tdemo item range first last] {
+		foreach S [$Tdemo item style set $I] C [$Tdemo column list] {
+		    if {$S eq $style} {
+			set match $I
+			break
+		    }
+		}
+		if {$match ne ""} break
+	    }
+	}
     }
     if {$match ne ""} {
 	set I $match
@@ -798,7 +836,7 @@ proc StyleEditor::StyleToCanvas {{scroll 0}} {
 	    $T selection clear
 	}
 	foreach state [$Tdemo item state get $I] {
-	    if {[lsearch -exact [$Tdemo state names] $state] != -1} {
+	    if {[lsearch -exact [$Tdemo item state names] $state] != -1} {
 		$T item state set root $state
 	    }
 	}
@@ -833,22 +871,37 @@ if 0 {
 	$T configure $name $current
     }
 }
-    set I root
-    $T item style set $I 0 $style
+    if {$domain eq "header"} {
+	set I first
+	$T header style set $I 0 $style
+    }
+    if {$domain eq "item"} {
+	set I root
+	$T item style set $I 0 $style
+    }
+
+    # Hack some minimum layout size > 0
+    foreach E [$T style elements $style] {
+	if {[scan [$T $domain bbox $I 0 $E] "%d %d %d %d" x1 y1 x2 y2] == 4} {
+	    if {$y2 - $y1 == 0 && $x2 - $x1 == 0} {
+		$T style layout $style $E -minwidth 10 -minheight 10
+	    }
+	}
+    }
 
     set scale 2
-    
+
     set dx 10
     set dy 10
 
-    scan [$T item bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
+    scan [$T $domain bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
     $canvas create rectangle \
 	[expr {$dx + $x1 * $scale}] [expr {$dy + $y1 * $scale}] \
 	[expr {$dx + $x2 * $scale}] [expr {$dy + $y2 * $scale}] \
 	-outline gray90
 
     foreach E [$T style elements $style] {
-	if {[scan [$T item bbox $I 0 $E] "%d %d %d %d" x1 y1 x2 y2] == 4} {
+	if {[scan [$T $domain bbox $I 0 $E] "%d %d %d %d" x1 y1 x2 y2] == 4} {
 	    $canvas create rectangle \
 		[expr {$dx + $x1 * $scale}] [expr {$dy + $y1 * $scale}] \
 		[expr {$dx + $x2 * $scale}] [expr {$dy + $y2 * $scale}] \
@@ -856,15 +909,18 @@ if 0 {
 	}
     }
 
-    scan [$T item bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
+    scan [$T $domain bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
     incr dy [expr {($y2 - $y1) * $scale + 20}]
 
     # Now give the style 1.5 times its needed space to test expansion
-    scan [$T item bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
-    $T column configure 0 -width [expr {($x2 - $x1) * 1.5}]
-    $T configure -itemheight [expr {($y2 - $y1) * 1.5}]
-
-    scan [$T item bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
+    scan [$T $domain bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
+    $T column configure 0 -width [expr {($x2 - $x1) * [Info scaleX]}]
+if 1 {
+    $T $domain configure $I -height [expr {($y2 - $y1) * [Info scaleY]}]
+} else {
+    $T configure -itemheight [expr {($y2 - $y1) * [Info scaleY]}]
+}
+    scan [$T $domain bbox $I 0] "%d %d %d %d" x1 y1 x2 y2
 
     $canvas create rectangle \
 	[expr {$dx + $x1 * $scale}] [expr {$dy + $y1 * $scale}] \
@@ -872,7 +928,7 @@ if 0 {
 	-outline gray90
 
     foreach E [$T style elements $style] {
-	if {[scan [$T item bbox $I 0 $E] "%d %d %d %d" x1 y1 x2 y2] == 4} {
+	if {[scan [$T $domain bbox $I 0 $E] "%d %d %d %d" x1 y1 x2 y2] == 4} {
 	    $canvas create rectangle \
 		[expr {$dx + $x1 * $scale}] [expr {$dy + $y1 * $scale}] \
 		[expr {$dx + $x2 * $scale}] [expr {$dy + $y2 * $scale}] \
@@ -908,3 +964,13 @@ proc StyleEditor::CanvasSelectElement {} {
     return
 }
 
+proc StyleEditor::ScaleX {value} {
+    Info scaleX [expr {$value / 100.0}]
+    StyleToCanvas
+}
+
+proc StyleEditor::ScaleY {value} {
+    Info scaleY [expr {$value / 100.0}]
+    StyleToCanvas
+}
+
